# -*- coding: utf-8 -*-
#
###
# Application: wah!cade
# File:        mplayer.py
# Description: MPlayer class
# Initial code: Jay Dolan <http://jdolan.dyndns.org/jaydolan/pymp.html>
# Copyright (c) 2006   Jay Dolan, Andy Balcombe <http://www.anti-particle.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

import os
import fcntl

class Mplayer:
    """Provides simple piped I/O to an mplayer process."""
    video_xid, mplayerIn, mplayerOut = None, None, None
    paused = False

    def __init__(self, video_xid, mute):
        """Initializes this MPlayer"""
        self.video_xid = video_xid
        self.mute = mute

    def play(self, video_filename):
        """Plays the specified video."""
        opts = ''
        if self.mute:
            opts = ' -nosound'
        #load player with specified vid & loop forever 
        #mpc = "mplayer -wid %s -slave -quiet -loop 0%s %s 2>/dev/null" % (
        mpc = "mplayer -wid %s -slave -quiet -loop 0%s %s >/dev/null 2>&1" % (
            self.video_xid, opts, video_filename)
        #open pipe
        self.mplayerIn, self.mplayerOut = os.popen2(mpc)
        fcntl.fcntl(self.mplayerOut, fcntl.F_SETFL, os.O_NONBLOCK)

    def cmd(self, command):
        """Issues command to mplayer."""
        if not self.mplayerIn:
            print "no mplayer for cmd!"
            return
        try:
            self.mplayerIn.write(command + "\n")
            #flush pipe
            self.mplayerIn.flush()
        except StandardError:
            return

    def pause(self):
        """Toggles pausing of the current mplayer job and status query."""
        if not self.mplayerIn:
            return
        if self.paused:
            #unpause
            self.paused = False
        else:
            #pause
            self.paused = True
        self.cmd("pause")

    def close(self):
        """Cleanly closes any IPC resources to mplayer."""
        if self.paused:
            #untoggle pause to cleanly quit
            self.pause()
        #ask mplayer to quit
        self.cmd("quit")
        #close pipes
        try:
            self.mplayerIn.close()
            self.mplayerOut.close()
        except StandardError:
            pass
        self.mplayerIn, self.mplayerOut = None, None
